/* Functions defined in keycvt.c.  This will go in the library, eventually */

#ifndef KeyUTiLsDOTh
#define KeyUTiLsDOTh

#include "key.h"
#define MAXNDIMKU 5

/* Use these as the 'order' argument in CellBBFromKey and GenerateKeys */
enum KeyOrder_e{ MORTON_ORDER=1, PH_ORDER };

#ifdef __cplusplus
extern "C"{
#endif /* __cplusplus */

typedef struct {
    int ndim;
    /* Store rmin and the size.  There are lots of choices.  Some might
       be better than others.  This is one of them... */
    float rmin[MAXNDIMKU];
    float sz[MAXNDIMKU];
} tbbox;			/* a tree-bbox,  */

void MakeBbox(tbbox *bb, int ndim, float *min, float *max);
void CornersBbox(const tbbox *bb, float *min, float *max);

/* CenterBbox is useful, nevertheless */
void CenterBbox(const tbbox *bb, float *center, float *sz);

/* This gives a tight bounding box around the list of positions.  Note
   that the result is rmin <= pos[i] and rmax >= pos[i].  The equality
   can be a headache for float-to-int conversions!  Consider using
   InflateBbox and or CubeBbox! */
void TightBbox(const float *pstart, int nobj, int pstride, int ndim, tbbox *bb);

/* Using MPMY_Combine generate the union over all processes of the given bb */
void MPMYCombineBbox(tbbox *bb);

/* Make the bbox a cube by expanding the smaller dimensions. */
void CubeBbox(tbbox *bb);

/* Increase the linear dimension by 'factor' on all sides */
void InflateBbox(tbbox *bb, float factor);

/* Return 1 if bb1 completely contains bb2 */
int ContainsBbox(const tbbox *bb1, const tbbox *bb2);

/* Construct bbu, the 'union' of bb1 and bb2 */
void UnionBbox(const tbbox *bb1, const tbbox *bb2, tbbox *bbu);

/* Generate keys for an array of positions (imagine sizeof(body) as the
   stride argument!) */
int GenerateKeys(const float *pstart, int nobj, int pstride, tbbox *bb, Key_t *kstart, int kstride, enum KeyOrder_e ordering);

Key_t KeyFromFloats(const float *pos, const tbbox *universe, int level, enum KeyOrder_e ordering);

/* Replacement for CellCorner:  you supply the bbox that describes the
   Universe, and we return a bbox that describes the cell */
void CellBBFromKey(Key_t key, tbbox *bb, tbbox *cellbb, enum KeyOrder_e ordering);

/* Some primitive building blocks.  They can be combined with
 KeyFromInts and IntsFromKey to do the full conversion... */
int IntsFromFloats(const float *x, unsigned int *ix, const tbbox *bb, int nbits);
void FloatsFromInts(const int *ix, float *x, const tbbox *bb, int nbits);

/* These two names conflict with physics_generic.c.  Good.  It will
   keep me from using physics_generic.c accidentally. */
Key_t KeyFromInts(unsigned int *xp, int ndim, int nbits);
int IntsFromKey(Key_t key, unsigned int *ip, int ndim);
Key_t PHKeyFromInts(unsigned int *xp, int ndim, int nbits);
int IntsFromPHKey(Key_t key, unsigned int *ip, int ndim);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif
